import { Injectable } from '@angular/core';
import { catchError, from, map, Observable, throwError } from 'rxjs';
import { Channel } from '../models/channel';
import { Message } from '../models/message';
import { HttpClient } from '@angular/common/http';
import { environment } from '../../environment/environment';
import { ListAvailableChannelsResponse } from './responses/chat';

@Injectable({
  providedIn: 'root'
})
export class ChatService {

  constructor(private http: HttpClient) { }

  public ListAvailableChannels(): Observable<Channel[]> {
    let url = `${environment.apiBase}/chat/channels`
    return this.http.get<ListAvailableChannelsResponse>(url, { withCredentials: true })
      .pipe(
        map(response => {
          if (response.error) {
            throw new Error(response.error);
          }

          if (response.channels) {
            return response.channels;
          }

          throw new Error("bad API response, missing channels with no error");
        }),
        catchError(error => throwError(() => new Error(error.error.message)))
      );
  }

  // TODO: implement
  public GetMessages(channelID: number): Observable<Message> {

    return from([
      {
        id: 1,
        sender: 'admin',
        channel: 1,
        time: new Date(),
        content: 'this is my first message'
      },
      {
        id: 1,
        sender: 'alice',
        channel: 1,
        time: new Date(),
        content: 'this is my first message'
      },
      {
        id: 1,
        sender: 'bob',
        channel: 1,
        time: new Date(),
        content: 'this is my first message'
      },
      {
        id: 1,
        sender: 'charlie',
        channel: 1,
        time: new Date(),
        content: 'this is my first message'
      },
      {
        id: 1,
        sender: 'admin',
        channel: 1,
        time: new Date(),
        content: 'this is my first message'
      },
      {
        id: 1,
        sender: 'admin',
        channel: 1,
        time: new Date(),
        content: 'this is my first message'
      },
      {
        id: 1,
        sender: 'admin',
        channel: 1,
        time: new Date(),
        content: 'this is my first message'
      },
      {
        id: 1,
        sender: 'admin',
        channel: 1,
        time: new Date(),
        content: 'this is my first message'
      },
      {
        id: 1,
        sender: 'admin',
        channel: 1,
        time: new Date(),
        content: 'this is my first message'
      },
      {
        id: 1,
        sender: 'admin',
        channel: 1,
        time: new Date(),
        content: 'this is my first message'
      },
      {
        id: 1,
        sender: 'admin',
        channel: 1,
        time: new Date(),
        content: 'this is my first message'
      },
      {
        id: 1,
        sender: 'admin',
        channel: 1,
        time: new Date(),
        content: 'this is my first message'
      },
      {
        id: 1,
        sender: 'admin',
        channel: 1,
        time: new Date(),
        content: 'this is my first message'
      },
      {
        id: 1,
        sender: 'admin',
        channel: 1,
        time: new Date(),
        content: 'this is my first message'
      },
      {
        id: 1,
        sender: 'admin',
        channel: 1,
        time: new Date(),
        content: 'this is my first message'
      },
      {
        id: 1,
        sender: 'admin',
        channel: 1,
        time: new Date(),
        content: 'this is my first message'
      },
      {
        id: 1,
        sender: 'admin',
        channel: 1,
        time: new Date(),
        content: 'this is my first message'
      },
      {
        id: 1,
        sender: 'admin',
        channel: 1,
        time: new Date(),
        content: 'this is my first message'
      },
      {
        id: 2,
        sender: 'admin',
        channel: 2,
        time: new Date(),
        content: 'this is my second message'
      }
    ]);
  }
}
